<?php if (!defined("_INSIDE")) die ("Restricted access!!!");
# This class is a helper to read log files of the DNET Proxy.
#
# (c)2008 under GPL by Steffen Ille <steffen@steffen-ille.de>
#                  and Joerg Risse  <TheWayne@the-hall.info>

require_once(realpath('./loader.inc.php'));
include_once(INC_PATH.'settings.inc.php');

class Cache {
  # Constructor - just to have one
  function Cache() {
  }

  # Function to read all files from a directory
  function readDirectory($path, $format="", $noDirectories=true) {
    clearstatcache();
    $arr = array();
    if (!@is_dir( $path )) { return $arr; }
    $handle = opendir( $path );
    while ($file = readdir($handle)) {
      if (($file <> ".") && ($file <> "..") && (false == $noDirectories || !is_dir($file))) {
        if (strlen($format) <= 0 || eregi($format,$file)) {
          $arr[] = $file;
        }
      }
    }
    closedir($handle);
    asort($arr);
    clearstatcache();
    return $arr;
  }

  # helper to get the name of an available cachefile
  function getFile($prj, $fileUnifier, $yesterday = false, $fileExt = "inc.php", $cachetimeID="std") {
    global $debug;

    if (empty($prj) || !is_bool($yesterday) || empty($fileExt)) return false;

    $cachefileType = (preg_match("/php/i",$fileExt)) ? "php" : "image";
    if ($cachefileType == "image" && empty($fileUnifier)) return false;
    
    $file_format = Cache::_createFileName($prj, $fileUnifier, $fileExt);
    if (empty($file_format)) return false;

    $file_format = str_replace("__DYN_CACHE__","([0-9]+)",quotemeta($file_format));

    // delete too old cached files
    Cache::_cleanUpOldFiles();

    $cache_filelist = Cache::readDirectory($GLOBALS['_dnet']['web']['cache']['path'],$file_format);
    $cachefile = false;
    $cachefileTime = (!empty($GLOBALS['_dnet']['web']['cache']['time'][$cachetimeID])) ? $GLOBALS['_dnet']['web']['cache']['time'][$cachetimeID] : $GLOBALS['_dnet']['web']['cache']['time']['std'];
    for($i=0; $i<count($cache_filelist); $i++) {
      // if cachefile (not php-file) was written in the last X seconds -> we use it to get stats-data
      if ($cachefileType == "image" && $cachefile === false && ($GLOBALS['_dnet']['act_time'] - filemtime($GLOBALS['_dnet']['web']['cache']['path'].$cache_filelist[$i])) <= $cachefileTime) {
        $cachefile = $GLOBALS['_dnet']['web']['cache']['path'].$cache_filelist[$i];
      }
      else if ($cachefileType == "php") {
        // if we are looking for php-include files - we need the newest of all.
        // -> old cachefiles are useful to avoid log-readings
        if ($cachefile === false) $cachefile = $GLOBALS['_dnet']['web']['cache']['path'].$cache_filelist[$i];
        else if (filemtime($GLOBALS['_dnet']['web']['cache']['path'].$cache_filelist[$i]) > filemtime($cachefile)) {
          @unlink($cachefile);
          $cachefile = $GLOBALS['_dnet']['web']['cache']['path'].$cache_filelist[$i];
        }
      }
      else { // cachefile too old or other cachefile found -> delete this file
        @unlink($GLOBALS['_dnet']['web']['cache']['path'].$cache_filelist[$i]);
      }
    }
    return $cachefile;
  }

  # helper to write a php array into a cachefile
  # returns filename of false
  function writePhp2File(&$data, $prj, $yesterday) {
    if (count($data) <= 0 || empty($prj) || !is_bool($yesterday)) return false;

    $cachefile = Cache::_createFileName($prj, "", "inc.php");
    if (!empty($cachefile)) {
      $cachefile = $GLOBALS['_dnet']['web']['cache']['path'].str_replace("__DYN_CACHE__",rand(100000,99999999),$cachefile);

      $txt = '<?php'."\n";
      $txt .= Cache::_parseArray2PhpCode($data, "d");
      $txt .= '?>';

      if (!Cache::_write2File($txt,$cachefile)) $cachefile = false;
      unset($txt);
    }
    return $cachefile;
  }

  # helper to write a PNG-image into cache-dir from an image-resource
  # returns filename of false
  function writeImage2PNG(&$image, $prj, $fileUnifier, $yesterday) {
    if (count($image) <= 0 || empty($prj) || !is_bool($yesterday)) return false;

    $cachefile = Cache::_createFileName($prj, $fileUnifier, ".png");
    if (!empty($cachefile)) {
      $cachefile = $GLOBALS['_dnet']['web']['cache']['path'].str_replace("__DYN_CACHE__",rand(100000,99999999),$cachefile);
      @imagePNG($image,$cachefile);
    }
    return $cachefile;
  }

  #########################
  # PRIVATE CLASS METHODS #
  #########################

  # private helper function to parse an array into PHP source code
  # needed to store cache-data into include-file
  function _parseArray2PhpCode(&$arr, $varname, $inner = false) {
    global $debug;
    if (!is_array($arr)) { return ""; }
    if (empty($varname)) { $varname = "d"; }

    if (!empty($debug)) { // Debug => better readable cache-data
      $txt = "";
      foreach($arr as $k => $v) {
        if (is_array($v)) { $txt .= Cache::_parseArray2PhpCode($v, ($varname."[".Cache::_parseQuoteVar($k)."]")); }
        else { $txt .= "\$".$varname."[".Cache::_parseQuoteVar($k)."]=".Cache::_parseQuoteVar($v).";\n"; }
      }
    } else { // not debugging => optimized cache-data
      $txt  = ((!$inner)?"\$".$varname."=array(":"");
      $txt1 = "";
      foreach($arr as $k => $v) {
        $txt1 .= ",".Cache::_parseQuoteVar($k)."=>";
        if (is_array($v)) { // recursive
          $txt1 .= "array(".substr(Cache::_parseArray2PhpCode($v, "", true),1).")";
        }
        else {
          $txt1 .= Cache::_parseQuoteVar($v);
        }
      }
      $txt.= ((!$inner)?substr($txt1,1):$txt1).((!$inner)?");\n":"");
    }
    return $txt;
  }

  # private helper function to quote strings
  # needed in Cache::parseArray2PhpCode()
  function _parseQuoteVar($var) {
    return (is_numeric($var))?$var:"'".$var."'";
  }

  # private helper function to write txt-based data into file
  function _write2File($txt_data,$filename) {
    if (empty($txt_data) || empty($filename)) return false;

    if ($fp = fopen( $filename, "w")) {
      fputs($fp, $txt_data, strlen($txt_data));
      fclose ($fp);
      return true;
    }
    return false;
  }

  # helper function to create a filename for a cachefile
  # -> filename contains a wildcard
  function _createFileName($prj, $fUnifier, $fExt) {
    if (empty($fUnifier)) $fUnifier = gmdate("Ymd",$GLOBALS['_dnet']['act_time']);
    if (empty($prj) || empty($fExt)) return false;
    if (strpos($fExt,".") == 0) $fExt = substr($fExt,1);
    return $prj.".".$fUnifier.".__DYN_CACHE__.".$fExt;
  }

  # helper function to delete old cache-files
  function _cleanUpOldFiles() {
    // search for all files in cache-directory
    clearstatcache();
    $arr = array();
    if (!@is_dir( $GLOBALS['_dnet']['web']['cache']['path'] )) { return false; }
    $handle = opendir( $GLOBALS['_dnet']['web']['cache']['path'] );
    while ($file = readdir($handle)) {
      if (($file <> ".") && ($file <> "..") && (false == $noDirectories || !is_dir($file))) {
        $arr[] = $GLOBALS['_dnet']['web']['cache']['path'].$file;
      }
    }
    closedir($handle);
    clearstatcache();

    // delete cache-files that are older then 1.5 days
    $del_time = time() - 129600;
    foreach($arr as $k=>$f) {
      if (fileatime($f) < $del_time) { @unlink($f); }
      clearstatcache();
    }
    return true;
  }

} # end: class Cache

?>