<?php if (!defined("_INSIDE")) die ("Restricted access!!!");
# This class is a helper to read log files of the DNET Proxy.
#
# (c)2008 under GPL by Steffen Ille <steffen@steffen-ille.de>
#                  and Joerg Risse  <TheWayne@the-hall.info>

require_once(realpath('./loader.inc.php'));
include_once(INC_PATH.'settings.inc.php');

class DNETLog {
  // Constructor -> mapped to standard function of this class
  function DNETLog($prj,$yesterday=false) {
    return DNETLog::readFile($prj,$yesterday);
  }

  # function to read in all Data from log-file
  function getData($prj,$yesterday=false)
  { if(empty($GLOBALS['_dnet']['project']) || empty($GLOBALS['_dnet']['project'][$prj]) || !is_bool($yesterday)) { return false; }

    // init start and end of the time-period we need from the log-files
    $start_time = gmmktime(0,0,0,gmdate("m",$GLOBALS['_dnet']['act_time']),gmdate("d",$GLOBALS['_dnet']['act_time']),gmdate("Y",$GLOBALS['_dnet']['act_time']));
    $end_time = (!$yesterday) ? gmdate($GLOBALS['_dnet']['act_time']) : gmmktime(23,59,59,gmdate("m",$GLOBALS['_dnet']['act_time']),gmdate("d",$GLOBALS['_dnet']['act_time']),gmdate("Y",$GLOBALS['_dnet']['act_time']));

    // try to get cache data (newest cache-file will be included)
    if (!empty($GLOBALS['_dnet']['web']['cache']['time']['std']) && class_exists("Cache")) {
      $cachefile = "";

      $cachefile = Cache::getFile($prj, "", $yesterday, "inc.php");
      if (!empty($cachefile)) { // we found a cachefile -> include data from it
        include_once($cachefile);
      }
    }

    // if we found a cachefile and had included cache-data
    if (!empty($d)) {
      if (!empty($d['upd'][1])) { // we got a end time of the cache data
        // -> we only need to read data from this point up to now
        $start_time = strtotime($d['upd'][1]." ".(($GLOBALS['_dnet']['act_offset'] >= 0)?"+".$GLOBALS['_dnet']['act_offset']:$GLOBALS['_dnet']['act_offset'])." sec"); // -> set as start-time
      }
    } else $d = array();

    // only read log files if we need todays data (these may change)
    // OR if no readed logs are stores in cache data
    // -> indicator of closed (no more changeable) cache-date for yesterday!
    if (!$yesterday || empty($d['h']) || isset($d['logs'])) {
      // get name of the proxy log file
      $log_file = DNETLog::getFilename($prj, $yesterday);
      if (DNETLog::_isHourlyLogRotation($prj, $yesterday)) {
        // if we have hourly log rotation, we (maybe) need to read more then 1 log file
        $log_file = eregi_replace(("\-".gmdate("H",$GLOBALS['_dnet']['act_time'])),"-".DNETLog::_getHourWildcard(gmdate("G",$start_time),gmdate("G",$end_time)),$log_file);
      }

      $logs2read = Cache::readDirectory($GLOBALS['_dnet']['web']['log_path'], $log_file, true);

      if (is_array($logs2read) && count($logs2read) > 0) {
        foreach($logs2read as $kl=>$l) DNETLog::_readLogFile($d, $l, array($start_time,$end_time));
      }
    }

    // we have cached data or read data from log for yesterday.
    // this data cant change and dont need to read again
    // -> store without $d['logs'] to inidcate that fact
    if ($yesterday && !empty($d['h']) && isset($d['logs'])) {
      unset($d['logs']);
      $d['read_new'] = true; // force storage of data into cache file
    }

    if (!empty($GLOBALS['_dnet']['web']['cache']['time']['std']) && class_exists("Cache")) { // lets write the data to cachefile
      if (!empty($d['read_new'])) { // hey man, did we got new data at all? -> if not, we dont need a new cache file
        unset($d['read_new']); // just to indicate if we have new data -> dont store it to cache file
        if (false !== Cache::writePHP2File($d, $prj, $yesterday) && !empty($cachefile)) {
          @unlink($cachefile);
        }
      }
    }

    return $d;
  }

  # function to get filename of the dnet-proxy log file
  function getFilename($prj, $yesterday = false) {
    global $debug;

    if (empty($GLOBALS['_dnet']['project']) || empty($GLOBALS['_dnet']['project'][$prj]) || !is_bool($yesterday) || ($yesterday && empty($GLOBALS['_dnet']['project'][$prj]['yesterday_log'])) || (!$yesterday && empty($GLOBALS['_dnet']['project'][$prj]['log']))) { return false; }

    // replace date-wildcard from log-filenames
    $logfile = ($yesterday) ? $GLOBALS['_dnet']['project'][$prj]['yesterday_log'] : $GLOBALS['_dnet']['project'][$prj]['log'];
    if(eregi("^(.*)__ACT_UTC_(.*)__(.*)$",$logfile,$logpart)) {
      $logfile = $logpart[1].gmdate($logpart[2],$GLOBALS['_dnet']['act_time']).$logpart[3];
    }
    return $logfile;
  }

  # helper function to get if we have hourly log-rotation or not
  function _isHourlyLogRotation($prj, $yesterday) {
    $logfile = ($yesterday) ? $GLOBALS['_dnet']['project'][$prj]['yesterday_log'] : $GLOBALS['_dnet']['project'][$prj]['log'];
    if(eregi("^(.*)__ACT_UTC_(.*)__(.*)$",$logfile,$logpart) && eregi("[HG]",$logpart[2])) return true;
    return false; 
  }
  
  # helper function to get a search-wildcard for a range of hours
  function _getHourWildcard($start_hour,$end_hour) {
    if (!is_numeric($start_hour) || !is_numeric($end_hour) || $start_hour < 0 || $end_hour < 0 || $start_hour > $end_hour || ($start_hour == 0 && $end_hour==23))
      return "[0-2]?[0-9]";

    if ($start_hour == $end_hour) return sprintf("%02d",$start_hour);

    $wildcard = "(";
    for($i=$start_hour;$i<=$end_hour;$i++) $wildcard .= (($i > $start_hour)?"|":"").sprintf("%02d",$i);
    $wildcard .= ")";

    return $wildcard;
  }

  # simple compare-function to sort client-data by blocks
  # used in methode: DNETLog::_readLogFile()
  function _cmp_clientRanking($a,$b)
  { if($a['sum'] == $b['sum']) { return 0; }
    return ($a['sum'] > $b['sum']) ? -1 : 1;
  }

  # helper function to make a arsort on a 2-dimensional array
  # - sorts array by values over one sub-dimension
  # used in methode: DNETLog::_readLogFile()
  function multi_arsort($arr)
  { $tmp = array();
    foreach ($arr as $k=>$v) { // build array for uasort
      foreach($v as $k1=>$v1) { // make 1 dimension from 2
        $tmp[($k."_".$k1)] = $v1;
      }
    }
    arsort($tmp);
    $s_arr = array();
    foreach($tmp as $k=>$v) { // split sorted array into prev. sub-dimensions
      $ks = split("_",$k);
      $s_arr[($ks[0])][($ks[1])] = $v;
    }
    unset($tmp);
    return $s_arr;
  }
  
  # helper function to read a log file into a data array
  function _readLogFile(&$ddata, $logfile, $timerange) {
    if (!isset($ddata) || !is_numeric($timerange[1]) || $timerange[1] < 0 || !is_numeric($timerange[0]) || $timerange[0] < 0 || $timerange[0] > $timerange[1] || empty($logfile) || !is_file($GLOBALS['_dnet']['web']['log_path'].$logfile)) return false;

    // if the size of the log file not changed since last read-access -> we dont need to read it again!
    if (!empty($ddata['logs'][$logfile]) && !empty($ddata['logs'][$logfile][0]) && filesize($GLOBALS['_dnet']['web']['log_path'].$logfile) == $ddata['logs'][$logfile][0]) return false;

    // start: read data from log
    $logdata = @file($GLOBALS['_dnet']['web']['log_path'].$logfile);
    $logdata_count = count($logdata);
    if(!is_array($logdata) || $logdata_count <= 0) { return false; }

    if (empty($ddata['sum']['blocks'])) $ddata['sum']['blocks'] = 0;
    if (empty($ddata['sum']['packages'])) $ddata['sum']['packages'] = 0;
    if (empty($ddata['upd'])) $ddata['upd'] = array();
    if (empty($ddata['logs'])) $ddata['logs'] = array();

    for($lc=((empty($ddata['logs'][$logfile]) || empty($ddata['logs'][$logfile][1]))?0:$ddata['logs'][$logfile][1]); $lc < $logdata_count; $lc++)
    { $tmp = explode(',',$logdata[$lc]);
      $tmp_time = strtotime($tmp[0]." ".(($GLOBALS['_dnet']['act_offset'] > 0)?"+".$GLOBALS['_dnet']['act_offset']:$GLOBALS['_dnet']['act_offset'])." sec");
      if(!is_array($GLOBALS['_dnet']['web']['excluded_ids']) || (!in_array($tmp[2],$GLOBALS['_dnet']['web']['excluded_ids']) && $tmp_time > $timerange[0] && $tmp_time <= $timerange[1]))
      { $dhour = intval(substr($tmp[0],11,2));
        $dcount= intval($tmp[4]);
        $ddata['sum']['blocks'] += $dcount;
        $ddata['sum']['packages'] += 1;
        $ddata['read_new'] = true;
        if(count($ddata['upd']) < 1) { $ddata['upd'][0] = $tmp[0]; }
        else { $ddata['upd'][1] = $tmp[0]; }

		if(!empty($ddata['h'][($dhour)])) { $ddata['h'][($dhour)] += $dcount; }
        else { $ddata['h'][($dhour)] = $dcount; }
        if(!isset($ddata['cl'][($tmp[2])]))
        { $ddata['cl'][($tmp[2])]['sum']['blocks'] = 0; // first entry for this client
          $ddata['cl'][($tmp[2])]['sum']['packages'] = 0;
          $ddata['cl'][($tmp[2])]['upd'][0] = $tmp[0];
        }
		$ddata['cl'][($tmp[2])]['sum']['packages'] += 1;
		$ddata['cl'][($tmp[2])]['sum']['blocks'] += $dcount;
        $ddata['cl'][($tmp[2])]['upd'][1] = $tmp[0];

        if(!empty($ddata['cl'][($tmp[2])]['ip'][($tmp[1])])) { $ddata['cl'][($tmp[2])]['ip'][($tmp[1])] += $dcount; }
        else { $ddata['cl'][($tmp[2])]['ip'][($tmp[1])] = $dcount; }
        if(!empty($ddata['cl'][($tmp[2])]['os'][($tmp[5])])) { $ddata['cl'][($tmp[2])]['os'][($tmp[5])] += $dcount; }
        else { $ddata['cl'][($tmp[2])]['os'][($tmp[5])] = $dcount; }
        if(!empty($ddata['cl'][($tmp[2])]['cpu'][($tmp[6])])) { $ddata['cl'][($tmp[2])]['cpu'][($tmp[6])] += $dcount; }
        else { $ddata['cl'][($tmp[2])]['cpu'][($tmp[6])] = $dcount; }
        if(!empty($ddata['cl'][($tmp[2])]['ver'][($tmp[7])])) { $ddata['cl'][($tmp[2])]['ver'][($tmp[7])] += $dcount; }
        else { $ddata['cl'][($tmp[2])]['ver'][($tmp[7])] = $dcount; }
        if(!empty($ddata['cl'][($tmp[2])]['core'][($tmp[6])][trim($tmp[8])])) { $ddata['cl'][($tmp[2])]['core'][($tmp[6])][trim($tmp[8])] += $dcount; }
        else { $ddata['cl'][($tmp[2])]['core'][($tmp[6])][trim($tmp[8])] = $dcount; }
        if(!empty($ddata['cl'][($tmp[2])]['h'][$dhour]['sum'])) { $ddata['cl'][($tmp[2])]['h'][$dhour]['sum'] += $dcount; }
        else { $ddata['cl'][($tmp[2])]['h'][$dhour]['sum'] = $dcount; }

        if(!empty($ddata['cl'][($tmp[2])]['h'][$dhour]['ip'][($tmp[1])])) { $ddata['cl'][($tmp[2])]['h'][$dhour]['ip'][($tmp[1])] += $dcount; }
        else { $ddata['cl'][($tmp[2])]['h'][$dhour]['ip'][($tmp[1])] = $dcount; }
        if(!empty($ddata['cl'][($tmp[2])]['h'][$dhour]['os'][($tmp[5])])) { $ddata['cl'][($tmp[2])]['h'][$dhour]['os'][($tmp[5])] += $dcount; }
        else { $ddata['cl'][($tmp[2])]['h'][$dhour]['os'][($tmp[5])] = $dcount; }
        if(!empty($ddata['cl'][($tmp[2])]['h'][$dhour]['cpu'][($tmp[6])])) { $ddata['cl'][($tmp[2])]['h'][$dhour]['cpu'][($tmp[6])] += $dcount; }
        else { $ddata['cl'][($tmp[2])]['h'][$dhour]['cpu'][($tmp[6])] = $dcount; }
        if(!empty($ddata['cl'][($tmp[2])]['h'][$dhour]['ver'][($tmp[7])])) { $ddata['cl'][($tmp[2])]['h'][$dhour]['ver'][($tmp[7])] += $dcount; }
        else { $ddata['cl'][($tmp[2])]['h'][$dhour]['ver'][($tmp[7])] = $dcount; }
        if(!empty($ddata['cl'][($tmp[2])]['h'][$dhour]['core'][($tmp[6])][trim($tmp[8])])) { $ddata['cl'][($tmp[2])]['h'][$dhour]['core'][($tmp[6])][trim($tmp[8])] += $dcount; }
        else { $ddata['cl'][($tmp[2])]['h'][$dhour]['core'][($tmp[6])][trim($tmp[8])] = $dcount; }
    } unset($tmp); } unset($logdata);
    foreach($ddata['cl'] as $id=>$cldata)
    { $ddata['cl'][$id]['upd'] = array_unique($ddata['cl'][$id]['upd']);
      natsort($ddata['cl'][$id]['upd']);
      $ddata['cl'][$id]['upd']=array($ddata['cl'][$id]['upd'][0],$ddata['cl'][$id]['upd'][1]);
      arsort($ddata['cl'][$id]['ip']);
      arsort($ddata['cl'][$id]['cpu']);
      arsort($ddata['cl'][$id]['os']);
      arsort($ddata['cl'][$id]['ver']);
      $ddata['cl'][$id]['core'] = DNETLog::multi_arsort($ddata['cl'][$id]['core']);
      for ($i=0; $i<24; $i++) {
        if (!empty($ddata['cl'][$id]['h'][$i])) {
          if (!empty($ddata['cl'][$id]['h'][$i]['ip'])) { arsort($ddata['cl'][$id]['h'][$i]['ip']); }
          if (!empty($ddata['cl'][$id]['h'][$i]['cpu'])) { arsort($ddata['cl'][$id]['h'][$i]['cpu']); }
          if (!empty($ddata['cl'][$id]['h'][$i]['os'])) { arsort($ddata['cl'][$id]['h'][$i]['os']); }
          if (!empty($ddata['cl'][$id]['h'][$i]['ver'])) { arsort($ddata['cl'][$id]['h'][$i]['ver']); }
          if (!empty($ddata['cl'][$id]['h'][$i]['core'])) { arsort($ddata['cl'][$id]['h'][$i]['core']); }
          if (!empty($ddata['cl'][$id]['h'][$i]['core'])) { $ddata['cl'][$id]['h'][$i]['core'] = DNETLog::multi_arsort($ddata['cl'][$id]['h'][$i]['core']); }
        }
      }
    } uasort($ddata['cl'],array("DNETLog","_cmp_clientRanking"));

    // store the position untill we already read the log file -> we dont need to read again ;)
    $ddata['logs'][$logfile] = array(filesize($GLOBALS['_dnet']['web']['log_path'].$logfile),$logdata_count);

    return true;
  }
} # end: class DNETLog

?>