<?php if (!defined("_INSIDE")) die ("Restricted access!!!");
# This class is a helper to dynamically generate HEX or RGB colorcodes.
# it is intented as feature-enhancement to the pie-chart script originally
# created by Rasmus - http://www.peters1.dk
#
# (c)2008 under GPL by Steffen Ille <steffen@steffen-ille.de>
#                  and Joerg Risse  <TheWayne@the-hall.info>


class DynColor {
  # Constructor -> mapped to standard function of this class
  # $mode -> "random" = random colors
  #          else = ordered colors (on a circle from red to red)
  function DynColor($mode, $colors=10,$grey_tones=true,$grey_only=false) {
    if (strtolower($mode) == "random") return DynColor::getRandomColors($colors, $grey_tones, $grey_only);
    else return DynColor::getColors($colors, $grey_tones, $grey_only);
  }

  # -----------------------------------------------------------------------------------------
  # PARAMETERS:
  # $HexColorString		(string)		color in hex (6 chars 0..9 & A..F)
  #
  # INPUT:
  # Example: color in 'hex' -> $input="FF0000" (= red)
  #
  # OUTPUT:
  # The result is an array containing $HexColorString different colors splitted in R,G,B values.
  # The array contains 3 elements 'r', 'g' and 'b'
  # Example: $output[0]['r']="255" $output[0]['g']="0"   $output[0]['b']="0"   (= red)
  #
  function hex2rgb($HexColorString) {
    return array('r'=>hexdec(substr($HexColorString, 0, 2)),'g'=>hexdec(substr($HexColorString, 2, 2)),'b'=>hexdec(substr($HexColorString, 4, 2)));
  }

  # -----------------------------------------------------------------------------------------
  # PARAMETERS:
  # $rgbColor		(array / string)		color in rgb as string (9 numbers) or array with 3 parts
  #
  # INPUT:
  # color as string -> $input="255000000" (= red)
  # coloar as array -> $input['r']="255" $input['g']="0" $input['b']="0"   (= red)
  #
  # OUTPUT:
  # color in 'hex' -> $input="FF0000" (= red)
  #
  function rgb2hex($rgbColor) {
    if (!is_array($rgbColor)) {
      $tmp_col = $rgbColor;
      $rgbColor['r'] = intval(substr($tmp_col, 0, 3));
      $rgbColor['g'] = intval(substr($tmp_col, 3, 3));
      $rgbColor['b'] = intval(substr($tmp_col, 6, 3));
    }
    return sprintf("%02X%02X%02X",$rgbColor['r'],$rgbColor['g'],$rgbColor['b']);
  }

  # -----------------------------------------------------------------------------------------
  # PARAMETERS:
  # $color	(array)			Array of colors to process. See INPUT annotation.
  # $code		('hex','rgb')	Codetype of the output. 'hex' => RRGGBB, 'rgb' => RRRGGGBBB
  #
  # INPUT:
  # The Input is a numbered array of colors splitted into values for R,G,B.
  # Example: 3 colors ->
  # $input[0]['r']="255" $input[0]['g']="0"   $input[0]['b']="0"   (= red)
  # $input[1]['r']="0"   $input[0]['g']="255" $input[0]['b']="0"   (= green)
  # $input[2]['r']="0"   $input[0]['g']="0"   $input[0]['b']="255" (= blue)
  #
  # OUTPUT:
  # The result is an array containing $colors formatted as $code color-code.
  # The array is numberded 0 ... $color-1
  # Example: 3 colors and $code='rgb' ->
  # $output[0]="255000000" $output[1]="000255000" $output[2]="000000255"
  # Example: 3 colors and $code='hex' ->
  # $output[0]="FF0000" $output[1]="00FF00" $output[2]="0000FF"
  function getCodes($color,$code='hex')
  { # return rgb-colorcodes INDEX -> RRRGGGBBB
    if($code=='rgb')
    { foreach($color as $idx=>$tmp){ $colorcode[$idx]=DynColor::hex2rgb($tmp); } }
    # return hex-colorcodes INDEX -> RRGGBB
    elseif($code=='hex')
    { foreach($color as $idx=>$tmp){ $colorcode[$idx]=DynColor::rcg2hex($tmp); } }
    if(count($colorcode)>0) { return $colorcode; } else { return false; }
  }

  # -----------------------------------------------------------------------------------------
  # PARAMETERS:
  # $colors		(int)			Number of different colors to return. Positive integer.
  # $grey_tones	(bool)			Use grayscale colors or not.
  # $grey_only	(bool)			Use only grayscale colors or not.
  #
  # OUTPUT:
  # The result is an array containing $color different colors splitted in R,G,B values.
  # The array is numberded 0 ... $color-1
  # Example: 3 colors ->
  # $output[0]['r']="255" $output[0]['g']="0"   $output[0]['b']="0"   (= red)
  # $output[1]['r']="0"   $output[1]['g']="255" $output[1]['b']="0"   (= green)
  # $output[2]['r']="0"   $output[2]['g']="0"   $output[2]['b']="255" (= blue)
  #
  function getRandomColors($colors=10,$grey_tones=true,$grey_only=false)
  { if($grey_only==true){ $fak=1; } elseif($grey_tones==true) { $fak=7; } else { $fak=6; }
    $runnings=ceil($colors/$fak); $stepwidth = 255/$runnings; $col=array(); $step=0; $i=0;
    while($i<$colors)
    { if($grey_only==false)
      { $col[$i]['r']=255; $col[$i]['g']=$step; $col[$i]['b']=0;       $i++; if($i==$colors){ break; } # red - yellow   (+)
        $col[$i]['r']=0; $col[$i]['g']=255-$step; $col[$i]['b']=255;   $i++; if($i==$colors){ break; } # cyan - blue    (-)
        $col[$i]['r']=0; $col[$i]['g']=255; $col[$i]['b']=$step;       $i++; if($i==$colors){ break; } # green - cyan   (+)
        $col[$i]['r']=255; $col[$i]['g']=0; $col[$i]['b']=255-$step;   $i++; if($i==$colors){ break; } # magenta - red  (-)
        $col[$i]['r']=$step; $col[$i]['g']=0; $col[$i]['b']=255;       $i++; if($i==$colors){ break; } # blue - magenta (+)
        $col[$i]['r']=255-$step; $col[$i]['g']=255; $col[$i]['b']=0;   $i++; if($i==$colors){ break; } # yellow - green (-)
      } if($grey_only==true || $grey_tones==true)
      { $col[$i]['r']=$step; $col[$i]['g']=$step; $col[$i]['b']=$step; $i++; if($i==$colors){ break; } # graytones      (+)
      } $step+=$stepwidth;
    } if(count($col)>0) { return $col; } else { return false; }
  }

  # -----------------------------------------------------------------------------------------
  # PARAMETERS:
  # $colors		(int)			Number of different colors to return. Positive integer.
  # $grey_tones	(bool)			Use grayscale colors or not.
  # $grey_only	(bool)			Use only grayscale colors or not.
  #
  # OUTPUT:
  # The result is an array containing $color different colors splitted in R,G,B values.
  # The array is numberded 0 ... $color-1
  # Example: 3 colors ->
  # $output[0]['r']="255" $output[0]['g']="0"   $output[0]['b']="0"   (= red)
  # $output[1]['r']="0"   $output[1]['g']="255" $output[1]['b']="0"   (= green)
  # $output[2]['r']="0"   $output[2]['g']="0"   $output[2]['b']="255" (= blue)
  #
  function getColors($colors=10,$grey_tones=true,$grey_only=false)
  { if($grey_only==true){ $fak=1; } elseif($grey_tones==true) { $fak=7; } else { $fak=6; }
    $runnings=ceil($colors/$fak); $stepwidth = 255/$runnings; $col=array(); $i=0; $step=0; $run=0;
    while($i<$colors)
    { if($grey_only==false)
      { while($run<$runnings) # red - yellow   (+)
        { $col[$i]['r']=255; $col[$i]['g']=$step; $col[$i]['b']=0;
          $i++; if($i==$colors){ break; } $run++; $step+=$stepwidth;
        } $step=0; $run=0; if($i==$colors){ break; }
        while($run<$runnings) # yellow - green (-)
        { $col[$i]['r']=255-$step; $col[$i]['g']=255; $col[$i]['b']=0;
          $i++; if($i==$colors){ break; } $run++; $step+=$stepwidth;
        } $step=0; $run=0; if($i==$colors){ break; }
        while($run<$runnings) # green - cyan   (+)
        { $col[$i]['r']=0; $col[$i]['g']=255; $col[$i]['b']=$step;
          $i++; if($i==$colors){ break; } $run++; $step+=$stepwidth;
        } $step=0; $run=0; if($i==$colors){ break; }
        while($run<$runnings) # cyan - blue    (-)
        { $col[$i]['r']=0; $col[$i]['g']=255-$step; $col[$i]['b']=255;
          $i++; if($i==$colors){ break; } $run++; $step+=$stepwidth;
        } $step=0; $run=0; if($i==$colors){ break; }
        while($run<$runnings) # blue - magenta (+)
        { $col[$i]['r']=$step; $col[$i]['g']=0; $col[$i]['b']=255;
          $i++; if($i==$colors){ break; } $run++; $step+=$stepwidth;
        } $step=0; $run=0; if($i==$colors){ break; }
        while($run<$runnings) # magenta - red  (-)
        { $col[$i]['r']=255; $col[$i]['g']=0; $col[$i]['b']=255-$step;
          $i++; if($i==$colors){ break; } $run++; $step+=$stepwidth;
        } $step=0; $run=0; if($i==$colors){ break; }
      }
      if($grey_only==true || $grey_tones==true)
      { while($run<$runnings) # graytones      (+)
        { $col[$i]['r']=$step; $col[$i]['g']=$step; $col[$i]['b']=$step;
          $i++; if($i==$colors){ break; } $run++; $step+=$stepwidth;
        } $step=0; $run=0; if($i==$colors){ break; }
    } } if(count($col)>0) { return $col; } else { return false; }
  }
  
} # end: class DynColor

?>